#__precompile__(true)
"""
OMRemote is used to remote control OpenModelica from Julia
"""
module OMRemote

  # (C) Christian Kral, 2017-2021, BSD-3 license
  # OMRemote remote access to run OpenModelica simulations from Julia
  # https://gitlab.com/christiankral/RemoteOM.git

  # v3.0.0 2021-04-20
  #     Switch to modularized function calls
  #     Remove sysVers parameter and indicate version number through sysLibs
  #         by using the @ spearator, e.g. "Modelica@3.2.3"
  #     Rename
  #         workDir => dir
  #         workFiles => files
  #     Changed default values
  #         copy = true => false
  #         dir = "/work" => "" # No default directory is used
  #     Also return OpenModelica omc handle as first argument
  #
  # v2.10.0
  #
  # v2.9.0

  using OMJulia
  using Base

  export envOM, simulateOM
  export loadFile, loadModel
  export instantiateModel

  include("loadModel.jl")
  include("loadFile.jl")
  include("instantiateModel.jl")
  include("simulate.jl")

  function envOM(env...)
    # Set environment variables
    if length(env) == 0
      println("    Setting environment for OpenModelica: OPENMODELICAHOME=/usr/local/bin")
      ENV["OPENMODELICAHOME"] = "/usr/local/bin"
    else
      println("    Setting environment for OpenModelica: OPENMODELICAHOME=" * env)
      ENV["OPENMODELICAHOME"] = env[1]
    end
  end

  """
  # Function call

  resFile = `simulateOM(model; sysLibs = "Modelica@3.2.3",
  files = "", dir = "", simDir = "/tmp/OpenModelica",
  StartTime = 0, StopTime = 0, Tolerance = 1E-6, copy = false)`

  # Description

  This function simulates and OpenModelica model and returns the file name
  of the result file

  # Variables

  `model` Modelica experiment to be simulated (incl. dot notation)

  `sysLibs` Modelica system libraries included by OpenModelica; default
            is the Modelica Standard Library (MLS) \"Modelica\"; several
            libraries may be separated by `:`; an additional version number
            is indicated by the '@' separator, e.g. \"Modelica@3.2.3\"

  `files` Modelica files to be loaded; if no path is provided files are
          loaded form `workDir`; otherwise the absolute file path has
          to be specified in `files` in order to read the file properly; several
          files may be separated by `:`

  `dir` Working directory; default value = "/work"

  `simDir` Simulation directory, default value = "/tmp/OpenModelica"

  `StartTime` Start time of simulation; default value = 0

  `StopTime` Stop time of simulation; if the StopTime <= StartTime,
             the original setting from the Experiment
             annotation is used; default value = 0

  `Tolerance` Simulation tolerance; default value = 1E-6

  `NumberOfIntervals` Number of output intervals; default value = 500

  `copy` Copy simulation result created by OpenModelica to directory `workDir`,
         if `true`; default value = false; in case of `true` the copied file
         name is assigned to the return variable `resFile`, otherwise the
         original file location created by OpenModelica is assigned to `resFile`
  """
  function simulateOM(model; sysLibs = "Modelica", files = "",
                      dir = "", simDir = "/tmp/OpenModelica",
                      StartTime = 0, StopTime = 0, Tolerance = 1E-6,
                      NumberOfIntervals = 500, copy = false)

        localDir = pwd()

        # Set environment variable for system libraries
        envOM()
        # Connect to OpenModelica
        omc = OMJulia.OMCSession()

        # Load system libraries
        for sysLib in split(sysLibs, ":")
            loadModel(omc, sysLib)
        end

        # Temporarely switch to work directory if dir is truly a directory
        if isdir(dir)
            cd(dir)
            OMJulia.sendExpression(omc, "cd(\"" * dir * "\")")
        end

        # Load Modelica files
        for file in split(files, ":")
            loadFile(omc, file)
        end

        # Create full path of simulation directory
        try
            mkpath(simDir)
            cd(simDir)
            OMJulia.sendExpression(omc, "cd(\"" * simDir * "\")")
        catch
            error("simulateOM: Could not create directory " * simDir)
        end
        # Instnatiate model
        instantiateModel(omc, model)

        simFile = simulate(omc, model; StartTime = StartTime,
            StopTime = StopTime, Tolerance = Tolerance,
            NumberOfIntervals = NumberOfIntervals)

        # Switch back to original directory before exiting at the next block
        cd(localDir)
        OMJulia.sendExpression(omc, "cd(\"" * localDir * "\")")
        # Copy result file to work directory if copy == true
        if simFile != false
            if copy
                if isfile(simFile)
                    pureName = splitpath(simFile)[end]
                    resFile = dir * "/" * pureName
                    try
                        println("    Copy file to:")
                        println("    " * resFile)
                        cp(simFile,resFile,force=true)
                        return (omc, resFile)
                    catch
                        println("    Could not copy file to " * resFile)
                        return (omc, simFile)
                    end
                    # Return result file name
                else
                    # Return result file name
                    error("simulateOM: File " * simFile * " not found")
                end
            else
                return (omc, simFile)
            end
        end
    end
end
